# Measurement script for Fristy by Attila Geresdi
# Based on the Measurement program Kun Zuo & Vincent Mourik
# Last updated: 18/01/2013

# We use this script for transport measurements, both DC and AC,
# as a function of 1 or 2 other variables. Its well suited to measure 
# with 1 or 2 Keithleys and/or 1 or 2 Lockins in 2 or 4 terminal geometries.
# If you are a first time user, we recommend scrolling down to the 
# 'initialization' part of the script, we put some usefull comments there.
# Comments/improvements are appreciated.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

from numpy import pi, random, arange, size, array, sin, cos, linspace, sinc, sqrt,log10,zeros,mean
from time import time, sleep
from shutil import copyfile
from os import mkdir
from os.path import exists
from lib.file_support.spyview import SpyView


import qt
import timetrack
import sys
import numpy as np
import data as d
import traces
import shutil
import os
import re #regular expression

keithley1 = qt.instruments.get('keithley1')
'''keithley2 = qt.instruments.get('keithley2')
keithley3 = qt.instruments.get('keithley3')
keithley4 = qt.instruments.get('keithley4')'''
lockin1 = qt.instruments.get('lockin1')
'''lockin2 = qt.instruments.get('lockin2')
#lockin3 = qt.instruments.get('lockin3')'''
ivvi = qt.instruments.get('IVVI')
triton 	= qt.instruments.get('triton')
#mw = qt.instruments.get('mw')
#agilent = qt.instruments.get('mwsource')

    
class majorana():
    
    def __init__(self):
        self.filename=filename
        self.generator=d.IncrementalGenerator(qt.config['datadir']+'\\'+self.filename,1);
    
    
    # Function generates data file, spyview file and copies the pyton script.
    def create_data(self,x_vector,x_coordinate,x_parameter,y_vector,y_coordinate,y_parameter,z_vector,z_coordinate,z_parameter):
        qt.Data.set_filename_generator(self.generator)
        data = qt.Data(name=self.filename)
        data.add_coordinate(x_parameter+' ('+x_coordinate+')',
                            size=len(x_vector),
                            start=x_vector[0],
                            end=x_vector[-1]) 
        data.add_coordinate(y_parameter+' ('+y_coordinate+')',
                            size=len(y_vector),
                            start=y_vector[0],
                            end=y_vector[-1]) 
        data.add_coordinate(z_parameter+' ('+z_coordinate+')',
                            size=len(z_vector),
                            start=z_vector[0],
                            end=z_vector[-1])
        data.add_value('Keithley 1/Gain')                    # Processed data
        data.add_value('Keithley 2/Gain')                        # Read out of Keithley 1
        data.add_value('Keithley 1')                        # Read out of Keithley 2
        data.add_value('Keithley 2')                        # Read out of Keithley 3
        data.add_value('Keithley 3')                        # Read out of Keithley 3
        #data.add_value('Keithley 1')                        # Read out of Keithley 2
        #data.add_value('Keithley 2')                        # Read out of Keithley 3
        data.add_value('Keithley 4')                          # Read out of Keithley 4
        data.add_value('Lockin_X')                          # Read out of Lockin 2
        data.add_value('Lockin_Y')
        data.add_value('conductance Keithley')
        data.add_value('conductance Lockin')                          # Read out of Lockin 3
        data.add_value('none')
        #data.add_value('Mixing Chamber')                    # Fridge temperature
        data.create_file()                                  # Create data file
        SpyView(data).write_meta_file()                     # Create the spyview meta.txt file
        traces.copy_script(sys._getframe().f_code.co_filename,data._dir,self.filename+'_'+str(self.generator._counter-1))           # Copy the python script into the data folder
        return data
       
    
    # Function reads out relevant data
    def take_data(self,dacx,x):
        
        ivvi.set(dacx,x)                                                            # Set specified dac to specified value, has to be done here because of delays needed for Lockin measurements
        
        if Nkeithleys == 0:
            
            if Nlockins ==0:
                print 'Why are you using this function if you dont want to read out any Keithley or Lockin...'
                qt.msleep(3.0)
                sys.exit()
            
            elif Nlockins == 1:
                qt.msleep(delay2)                                                    # Use explained at bottom of the script
                L1 = lockin1.get_X()                                                 # Read out Lockin1
                value = (L1/GainL1)/amplitude                                        # Process the data
                datavalues = [value,0,0,L1,0]
                qt.msleep(0.01)
            
            elif Nlockins == 2:
                qt.msleep(delay2)                                                    # Use explained at bottom of the script
                L1 = lockin1.get_X()                                                 # Read out Lockin1
                L2 = lockin2.get_X()                                                 # Read out Lockin2
                value = (L1/GainL1)*(GainL2/L2)                                      # Process the data
                datavalues = [value,0,0,L1,L2]
                qt.msleep(0.01)
        
        elif Nkeithleys == 1:
            
            if Nlockins == 0:
                qt.msleep(delay2) 
                K1 = keithley1.get_readlastval()                                     # Read out Keithley1            
                value = K1/GainK1                                                    # Process the data
                datavalues = [value,K1,0,0,0]
                qt.msleep(0.02)
            
            elif Nlockins == 1:
                qt.msleep(delay2)                                                    # Use explained at bottom of the script
                K1 = keithley1.get_readlastval()                                     # Read out Keithley1
                L1 = lockin1.get_X()                                                 # Read out Lockin1
                value = (L1/GainL1)/amplitude                                    # Process the data
                datavalues = [value,K1,L1,0,0,0,0]
                qt.msleep(0.02)
            
            elif Nlockins == 2:
                qt.msleep(delay2)                                                    # Use explained at bottom of the script
                K1 = keithley1.get_readlastval()                                     # Read out Keithley1
                L1_X = lockin1.get_X()                # Read out Lockin1
                L1_Y = lockin1.get_Y()
                L2_X = lockin2.get_X()                                                 # Read out Lockin2
                L2_Y = lockin2.get_Y()
                if L2_X == 0:
                    value = 1e9
                else:
                    value = (L1_X/GainL1)/L2_X                                      # Process the data
                datavalues = [value,K1,0,L1_X,L1_Y,L2_X,L2_Y]
                qt.msleep(0.02)
            
        elif Nkeithleys == 2:
            
            if Nlockins == 0:
                qt.msleep(delay2)
                K1 = keithley1.get_readlastval()                                     # Read out Keithley1
                K2 = keithley2.get_readlastval()                                     # Read out Keithley2
                value2 = K2/GainK2                                                   # Processed data
                value1 = K1/GainK1                                                  # Process the data
                #value = K1/GainK1/dcvoltage
                #if K1 == 0:
                #    value = 1e9
                #else:
                value = value2/value1
                # if value == 0:
                    # Rvalue = 1e9
                # else:
                    # Rvalue = 1/value
                #t_mc = float(triton.get_MCTemp())
                datavalues = [value1,value2,K1,K2,0,0,0,0,0,0,0]
                #qt.msleep(0.02)
            
            elif Nlockins == 1:
                qt.msleep(delay2)                                                    # Use explained at bottom of the script
                K1 = keithley1.get_readlastval()                                     # Read out Keithley1
                K2 = keithley2.get_readlastval()                                     # Read out Keithley2
                L1_X = lockin1.get_X()                                                 # Read out Lockin1
                L1_Y = lockin1.get_Y()                                                 # Read out Lockin1
                value = (L1_X/GainL1)/(amplitude*Vrange/100)                                        # Process the data
                #t_mc = float(triton.get_MCTemp())
                datavalues = [value,K1,K2,L1_X,L1_Y,0,0,0,0,0,0]
                qt.msleep(0.02)
            
            elif Nlockins == 2:
                qt.msleep(delay2)                                                    # Use explained at bottom of the script
                K1 = keithley1.get_readlastval()                                     # Read out Keithley1
                K2 = keithley2.get_readlastval()                                     # Read out Keithley2
                L1_X = lockin1.get_X()                                                 # Read out Lockin1_X
                L1_Y = lockin1.get_Y()
                L2_X = lockin2.get_X()                                                 # Read out Lockin2_X
                L2_Y = lockin2.get_Y()
                if L2_X == 0:
                    value = 1e9
                else:
                    value = (L1_X/GainL1)/(L2_X/GainL2)                                      # Process the data
                t_mc = float(triton.get_MCTemp())
                datavalues = [value,K1,K2,L1_X,L1_Y,L2_X,L2_Y,t_mc]
                qt.msleep(0.02)
                
            elif Nlockins == 3:
                qt.msleep(delay2)
                
                L1_X = lockin1.get_X()                                                 # Read out Lockin1
                L1_Y = lockin1.get_Y()
            
                L2_X = lockin2.get_X()                                                 # Read out Lockin2
                L2_Y = lockin2.get_Y()			
            
                K1 = keithley1.get_readlastval()
                K2 = keithley2.get_readlastval()
                #K3 = keithley3.get_readlastval()	
                L3_X = lockin3.get_X()                                                 # Read out Lockin3
                L3_Y = lockin3.get_Y()

                #R=L1_X/GainL1/(L3_X)
                #R2=L2_X/GainL2/(L3_X)
            
                #Rxx = (L1_X/GainL1)/self.applied_current_bias                                           # Process the data
                #Rxy = 0 #(L2_X/GainL2)/self.applied_current_bias
			
                # L3_X = self.applied_current_bias
                # L3_Y = 0
			
                if L3_X !=0: #K2 != 0: 
                    #R1 = 0.5e-3*1e6/K1
                    #R1 = K1/K2*1e7/100                                           # Process the data
                    Rxx = L1_X/L3_X*GainL3/GainL1
                    #R2 = K1/K3*1e6/100
                                        
                    Rxy = L2_X/L3_X*GainL3/GainL2 
                else:
                    Rxx = 1e7
                    #R2 = 1e7
                    Rxy = 0



                if Rxx != 0:
                    Gxx = 12906/Rxx
                    Gd = 12906/(Rxx+abs(Rxy))
                else:
                    Gxx = 1e5
            
                #K1=0
                #K2=0
                #if Nkeithleys == 2:
                #    K1 = keithley1.get_readlastval()
                #    K2 = keithley2.get_readlastval()
                #R=K2/GainK2/self.applied_current_bias
            
                #datavalues = [R,R2,K1,0,0,L1_X,L1_Y,L2_X,L2_Y,L3_X,L3_Y]
                datavalues = [Rxx,Rxy,K1,Gd,Gxx,L1_X,L1_Y,L2_X,L2_Y,L3_X,L3_Y]
                #datavalues = [R1,0,K1,K2,0,L1_X,L1_Y,L2_X,L2_Y,0,0]
                #print "Rxx: ",Rxx,"\t Rxy: ",Rxy
                qt.msleep(0.01)
        
        elif Nkeithleys == 3:
            
            if Nlockins == 0:
                qt.msleep(delay2)
                K1 = keithley1.get_readlastval()                                     # Read out Keithley1 (current)
                K2 = keithley2.get_readlastval()                                     # Read out Keithley2  (voltage)
                K3 = keithley3.get_readlastval()                                     # Read out Keithley3 (Leakage TG)
                value2 = K2/GainK2                                                   #
                value1 = K1/GainK1                                                  # Process the data
                #value = K1/GainK1/dcvoltage
                #if K1 == 0:
                #    value = 1e9
                #else:
                value = value2/value1
                # if value == 0:
                    # Rvalue = 1e9
                # else:
                    # Rvalue = 1/value
                #t_mc = float(triton.get_MCTemp())
                datavalues = [value1,value2,K1,K2,K3,value,0,0,0,0,0]
                #qt.msleep(0.02)
            
            if Nlockins == 1:
                qt.msleep(delay2)
                K1 = keithley1.get_readlastval()                                     # Read out Keithley1 (current)
                K2 = keithley2.get_readlastval()                                     # Read out Keithley2  (voltage)
                K3 = keithley3.get_readlastval()                                     # Read out Keithley3 (Leakage TG)
                value2 = K2/GainK2                                                   #
                value1 = K1/GainK1                                                  # Process the data
                #value = K1/GainK1/dcvoltage
                #if K1 == 0:
                #    value = 1e9
                #else:
                value = value2/value1
                # if value == 0:
                    # Rvalue = 1e9
                # else:
                    # Rvalue = 1/value
                #t_mc = float(triton.get_MCTemp())
                L1_X = lockin1.get_X()                                                 # Read out Lockin1
                L1_Y = lockin1.get_Y()                                                 # Read out Lockin1
                value = (L1_X/GainL1)/(amplitude*Vrange/100)                                        # Process the data
                datavalues = [value1,value2,K1,K2,K3,L1_X,L1_Y,0,0,0,0]                           
                qt.msleep(0.02)
            if Nlockins == 2:
                qt.msleep(delay2)
                K1 = keithley1.get_readlastval()                                     # Read out Keithley1 (current)
                K2 = keithley2.get_readlastval()                                     # Read out Keithley2  (voltage)
                K3 = keithley3.get_readlastval()                                     # Read out Keithley3 (Leakage TG)
                value2 = K2/GainK2                                                   #
                value1 = K1/GainK1                                                  # Process the data
                #value = K1/GainK1/dcvoltage
                #if K1 == 0:
                #    value = 1e9
                #else:
                value = value2/value1
                # if value == 0:
                    # Rvalue = 1e9
                # else:
                    # Rvalue = 1/value
                #t_mc = float(triton.get_MCTemp())
                L1_X = lockin1.get_X()                                                 # Read out Lockin1
                L1_Y = lockin1.get_Y()                                                 # Read out Lockin1
                L2_X = lockin2.get_X()                                                 # Read out Lockin1
                L2_Y = lockin2.get_Y()                                                 # Read out Lockin1
                value = (L1_X/GainL1)/(amplitude*Vrange/100)                                        # Process the data
                datavalues = [value1,value2,K1,K2,K3,L1_X,L1_Y,L2_X,L2_Y,0,0]                           
                qt.msleep(0.02)
        elif Nkeithleys ==4:
        
            if Nlockins == 0:
                K1 = keithley1.get_readlastval()                                     # Read out Keithley1 (current)
                K2 = keithley2.get_readlastval()                                     # Read out Keithley2  (voltage)
                K3 = keithley3.get_readlastval()                                     # Read out Keithley3 (Leakage TG)
                K4 = keithley4.get_readlastval()                                     # Read out Keithley3 (Leakage BG)                
                value2 = K2/GainK2                                                   #
                value1 = K1/GainK1                                                  # Process the data
                #value = K1/GainK1/dcvoltage
                #if K1 == 0:
                #    value = 1e9
                #else:
                value = value2/value1
                # if value == 0:
                    # Rvalue = 1e9
                # else:
                    # Rvalue = 1/value
                #t_mc = float(triton.get_MCTemp())
                datavalues = [value1,value2,K1,K2,K3,K4,value,0,0,0,0]
                #qt.msleep(0.02)            
                qt.msleep(delay2)
                
            if Nlockins == 1:
                qt.msleep(delay2)
                K1 = keithley1.get_readlastval()                                     # Read out Keithley1 (current)
                K2 = keithley2.get_readlastval()                                     # Read out Keithley2  (voltage)
                K3 = keithley3.get_readlastval()                                     # Read out Keithley3 (Leakage TG)
                K4 = keithley4.get_readlastval()                                     # Read out Keithley3 (Leakage BG) 
                value2 = K2/GainK2                                                   #
                value1 = K1/GainK1                                                  # Process the data
                #value = K1/GainK1/dcvoltage
                #if K1 == 0:
                #    value = 1e9
                #else:
                value = value1/value2
                # if value == 0:
                    # Rvalue = 1e9
                # else:
                    # Rvalue = 1/value
                #t_mc = float(triton.get_MCTemp())
                L1_X = lockin1.get_X()                                                 # Read out Lockin1
                L1_Y = lockin1.get_Y()                                                 # Read out Lockin1
                value_Lockin = (L1_X/GainL1)/(amplitude*Vrange/100)                    # Process the data
                datavalues = [value1,value2,K1,K2,K3,K4,L1_X,L1_Y,value,value_Lockin,0]                           
                qt.msleep(0.02)
        else:
            print 'You try to run a too complicated experiment...'
            qt.msleep(3.0)
            sys.exit()
        return datavalues
        qt.msleep(0.01)                                                            # Keep GUI responsive
    # Function to read out temperature
    # TODO: readout of AVS
    def read_T(self):
        try: 
            t_mc = float(triton.get_MCTemp())
        except:
            t_mc=0
        
        return t_mc
    
    ################ 1D scans #####################    
    
    # 1D sweep of a single dac
    def _single_dac_sweep(self,xname,dacx,xstart,xend,xsteps):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        y_vector = [0]
        z_vector = [0]
        
        data = self.create_data(x_vector,xname,dacx,y_vector,'none','y_parameter',z_vector,'none','z_parameter')                                # create data file, spyview metafile, copy script
        
        for x in x_vector:
            datavalues = self.take_data(dacx,x)                                                                                                 # Go to next sweep value and take data
            T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature
            data.add_data_point(x,0,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],T_mc,0,0,0,0,0)                               # write datapoint into datafile

        
        data.new_block()
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()
    
    
    # Repeatedly sweep a single dac n times, n = repetions
    def _repeat_single_dac_sweep(self,xname,dacx,xstart,xend,xsteps,repetions):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        y_vector = linspace(1,repetions,repetions)                                                                                              # this sweep variable is defined to enable 2D plotting in spyview
        z_vector = [0]
        
        data = self.create_data(x_vector,xname,dacx,y_vector,'n','repetions',z_vector,'none','z_parameter')                                     # create data file, spyview metafile, copy script
        
        counter = 0
        T_init = timetrack.time()
        T_now = 0
        
        for i in arange(repetions):
            [starttime, counter] = timetrack.start(counter)
            tstart = timetrack.time()
            ivvi.set(dacx,x_vector[0])
            T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature
            qt.msleep(delay1)                                                                                                                   # use explained at the bottom of the script
          
            for x in x_vector:
                datavalues = self.take_data(dacx,x)                                                                                             # Go to next sweep value and take data
                data.add_data_point(x,0,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],T_now)  # Note that the current time is stored in the T_mc column                         # write datapoint into datafile
            
            timetrack.remainingtime(starttime,repetions,counter)                                                                                # Calculate and print remaining scantime
            data.new_block()
            T_now = timetrack.time()-T_init

        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()    
    
    
    # 1D sweep of 2 dac's simultaneously; sweep vectors can have arbitrary start and end values.
    def _2dacs_sweep(self,xname,dacx,xstart,xend,mname,dacm,mstart,mend,steps):
        qt.mstart()
                
        # Create sweep vectors
        x_vector = linspace(xstart,xend,steps+1)
        y_vector = [0]
        z_vector = [0]
        
        data = self.create_data(x_vector,xname,dacx,y_vector,'none','y_parameter',z_vector,'none','z_parameter')                               # create data file, spyview metafile, copy script
        
        # Define sweep vector for other dac
        m_vector = linspace(mstart,mend,steps+1)
        
        for i in arange(len(x_vector)):
            x = x_vector[i]
            m = m_vector[i]
            ivvi.set(dacm,m)                                                                                                                    # Other dac needs to be set before running take_data
            datavalues = self.take_data(dacx,x)                                                                                                 # Go to next sweep value and take data
            #T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature
            data.add_data_point(x,0,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],datavalues[5],datavalues[6],datavalues[7],datavalues[8],datavalues[9],0)                               # write datapoint into datafile
        
        data.new_block()
        data._write_settings_file()
        data.close_file()
        qt.mend()


    # 1D sweep of 3 dac's simultaneously; sweep vectors can have arbitrary start and end values.
    def _3dacs_sweep(self,xname,dacx,xstart,xend,mname,dacm,mstart,mend,nname,dacn,nstart,nend,steps):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,steps+1)
        y_vector = [0]
        z_vector = [0]
        
        data = self.create_data(x_vector,xname,dacx,y_vector,'none','y_parameter',z_vector,'none','z_parameter')                               # create data file, spyview metafile, copy script
        
        # Define sweep vector for other dacs
        m_vector = linspace(mstart,mend,steps+1)
        n_vector = linspace(nstart,nend,steps+1)
        
        for i in arange(len(x_vector)):
            x = x_vector[i]
            m = m_vector[i]
            n = n_vector[i]
            ivvi.set(dacm,m)                                                                                                                    # Other dacs needs to be set before running take_data
            ivvi.set(dacn,n)
            datavalues = self.take_data(dacx,x)                                                                                                 # Go to next sweep value and take data
            T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature
            data.add_data_point(x,0,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],T_mc)                               # write datapoint into datafile
        
        data.new_block()
        data._write_settings_file()
        data.close_file()
        qt.mend()
    
    
    # 1D sweep of 4 dac's simultaneously; sweep vectors can have arbitrary start and end values.
    def _4dacs_sweep(self,xname,dacx,xstart,xend,mname,dacm,mstart,mend,nname,dacn,nstart,nend,pname,dacp,pstart,pend,steps):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,steps+1)
        y_vector = [0]
        z_vector = [0]
        
        data = self.create_data(x_vector,xname,dacx,y_vector,'none','y_parameter',z_vector,'none','z_parameter')                               # create data file, spyview metafile, copy script
        
        # Define sweep vector for other dacs
        m_vector = linspace(mstart,mend,steps+1)
        n_vector = linspace(nstart,nend,steps+1)
        p_vector = linspace(pstart,pend,steps+1)
        
        for i in arange(len(x_vector)):
            x = x_vector[i]
            m = m_vector[i]
            n = n_vector[i]
            p = p_vector[i]
            ivvi.set(dacm,m)                                                                                                                    # Other dacs needs to be set before running take_data
            ivvi.set(dacn,n)
            ivvi.set(dacp,p)
            datavalues = self.take_data(dacx,x)                                                                                                 # Go to next sweep value and take data
            T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature
            data.add_data_point(x,0,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],T_mc)                               # write datapoint into datafile
        
        data.new_block()
        data._write_settings_file()
        data.close_file()
        qt.mend()
        
        # 1D sweep of 5 dac's simultaneously; sweep vectors can have arbitrary start and end values.
    def _5dacs_sweep(self,xname,dacx,xstart,xend,mname,dacm,mstart,mend,nname,dacn,nstart,nend,pname,dacp,pstart,pend,qname,dacq,qstart,qend,steps):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,steps+1)
        y_vector = [0]
        z_vector = [0]
        
        data = self.create_data(x_vector,xname,dacx,y_vector,'none','y_parameter',z_vector,'none','z_parameter')                               # create data file, spyview metafile, copy script
        
        # Define sweep vector for other dacs
        m_vector = linspace(mstart,mend,steps+1)
        n_vector = linspace(nstart,nend,steps+1)
        p_vector = linspace(pstart,pend,steps+1)
        q_vector = linspace(qstart,qend,steps+1) 
        
        for i in arange(len(x_vector)):
            x = x_vector[i]
            m = m_vector[i]
            n = n_vector[i]
            p = p_vector[i]
            q = q_vector[i]
            ivvi.set(dacm,m)                                                                                                                    # Other dacs needs to be set before running take_data
            ivvi.set(dacn,n)
            ivvi.set(dacp,p)
            ivvi.set(dacq,q)
            datavalues = self.take_data(dacx,x)                                                                                                 # Go to next sweep value and take data
            T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature
            data.add_data_point(x,0,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],T_mc)                               # write datapoint into datafile
        
        data.new_block()
        data._write_settings_file()
        data.close_file()
        qt.mend()

        
        # 1D sweep of 11 dacs, all together, same range
    def _11_dacs_sweep(self,xname,dacx,yname,dacy,zname,dacz,kname,dack,lname,dacl,mname,dacm,nname,dacn,oname,daco,pname,dacp,qname,dacq,rname,dacr,xstart,xend,xsteps):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        y_vector = [0]
        z_vector = [0]
        
        data = self.create_data(x_vector,xname,dacx,y_vector,'none','y_parameter',z_vector,'none','z_parameter')                                # create data file, spyview metafile, copy script
        
        for x in x_vector:
            ivvi.set(dacy,x)
            ivvi.set(dacz,x)
            ivvi.set(dack,x)
            ivvi.set(dacl,x)
            ivvi.set(dacm,x)
            ivvi.set(dacn,x)
            ivvi.set(daco,x)
            ivvi.set(dacp,x)
            ivvi.set(dacq,x)
            ivvi.set(dacr,x)
            datavalues = self.take_data(dacx,x)                                                                                                 # Go to next sweep value and take data
            T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature
            data.add_data_point(x,0,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],T_mc)                               # write datapoint into datafile
        
        data.new_block()
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()
        
        
    #################### 2D scans ####################
    
    # 2D scan of one dac vs another one
    def _dac_vs_dac(self,xname,dacx,xstart,xend,xsteps,yname,dacy,ystart,yend,ysteps):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        y_vector = linspace(ystart,yend,ysteps+1)
        z_vector = [0]
        
        data = self.create_data(x_vector,xname,dacx,y_vector,yname,dacy,z_vector,'none','z_parameter')                                          # create data file, spyview metafile, copy script
        
        counter = 0
        
        for y in y_vector:
            [starttime, counter] = timetrack.start(counter)
            tstart = timetrack.time()
            ivvi.set(dacy,y)
            ivvi.set(dacx,x_vector[0])
            T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature
            qt.msleep(delay1)                                                                                                                   # use explained at the bottom of the script
            
            for x in x_vector:
                datavalues = self.take_data(dacx,x)                                                                                             # Go to next sweep value and take data
                data.add_data_point(x,y,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],0,0,T_mc,0,0,0)                           # write datapoint into datafile

            timetrack.remainingtime(starttime,ysteps+1,counter)                                                                                 # Calculate and print remaining scantime
            data.new_block()
            
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()
        
    def _dac_vs_dac_splitted_range(self,xname,dacx,xstart,xend,xsteps,xstart2,xend2,xsteps2,yname,dacy,ystart,yend,ysteps):
        qt.mstart()
        
        # Create sweep vectors
        x_vector1 = linspace(xstart,xend,xsteps+1)
        x_vector2 = linspace(xstart2,xend2,xsteps2+1)
        
        x_vector = concatenate([x_vector1,x_vector2])        
        y_vector = linspace(ystart,yend,ysteps+1)
        z_vector = [0]
        
        data = self.create_data(x_vector,xname,dacx,y_vector,yname,dacy,z_vector,'none','z_parameter')                                          # create data file, spyview metafile, copy script
        
        counter = 0
        
        for y in y_vector:
            [starttime, counter] = timetrack.start(counter)
            tstart = timetrack.time()
            ivvi.set(dacy,y)
            ivvi.set(dacx,x_vector[0])
            T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature 
            qt.msleep(delay1)                                                                                                                   # use explained at the bottom of the script
            
            for x in x_vector:
                datavalues = self.take_data(dacx,x)                                                                                             # Go to next sweep value and take data
                data.add_data_point(x,y,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],T_mc)                           # write datapoint into datafile
            
            timetrack.remainingtime(starttime,ysteps+1,counter)                                                                                 # Calculate and print remaining scantime
            data.new_block()
            
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()

    def _dac_vs_rfpower(self,xname,dacx,xstart,xend,xsteps,freq,ystart,yend,ysteps,db_on):
        qt.mstart()
         
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        y_vector = linspace(ystart,yend,ysteps+1)
        z_vector = [0]
        
        def db2amp(db):
            amp = 10.0**(db/20.0)
            return amp
        def amp2db(amp):
            db = 20*log10(amp)
            return db
            
        if db_on != 1:
            ystart = db2amp(ystart)
            yend = db2amp(yend)
            y_vector_lin = linspace(ystart,yend,ysteps+1)
            y_vector = amp2db(y_vector_lin)
            data = self.create_data(x_vector,xname,dacx,y_vector,'rfpower','sqrt(P)',z_vector,'none','z_parameter')  
        else:              
            data = self.create_data(x_vector,xname,dacx,y_vector,'rfpower','dBm',z_vector,'none','z_parameter')                                          # create data file, spyview metafile, copy script
        
        counter = 0
        
        agilent.set_frequency(freq)
        #agilent.set_power(-90.0)
        
        agilent.set_status('on')
        for y in y_vector:
            [starttime, counter] = timetrack.start(counter)
            tstart = timetrack.time()
            #ivvi.set(dacy,y)
            #ivvi.set(dacx,x_vector[0])
            agilent.set_power(y)
            
            T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature 
            qt.msleep(delay1)                                                                                                                   # use explained at the bottom of the script
            
            for x in x_vector:
                datavalues = self.take_data(dacx,x)                # Go to next sweep value and take data
                data.add_data_point(x,y,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],datavalues[5],datavalues[6],0,0,0,0)                           # write datapoint into datafile
            
            timetrack.remainingtime(starttime,ysteps+1,counter)                                                                                 # Calculate and print remaining scantime
            data.new_block()
        
        agilent.set_status('off')
        
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()

    def _dac_vs_rffreq(self,xname,dacx,xstart,xend,xsteps,power,ystart,yend,ysteps):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        y_vector = linspace(ystart,yend,ysteps+1)
        z_vector = [0]
        
        data = self.create_data(x_vector,xname,dacx,y_vector,'rffreq','Hz',z_vector,'none','z_parameter')                                          # create data file, spyview metafile, copy script
        
        counter = 0
        
        agilent.set_power(power)
        
        
        agilent.set_status('on')
        
        for y in y_vector:
            [starttime, counter] = timetrack.start(counter)
            tstart = timetrack.time()
            #ivvi.set(dacy,y)
            #ivvi.set(dacx,x_vector[0])
            agilent.set_frequency(y)
            
            T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature 
            qt.msleep(delay1)                                                                                                                   # use explained at the bottom of the script
            
            for x in x_vector:
                datavalues = self.take_data(dacx,x)                                                                                             # Go to next sweep value and take data
                data.add_data_point(x,y,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],datavalues[5],datavalues[6],0,0,0,0)                           # write datapoint into datafile
            
            timetrack.remainingtime(starttime,ysteps+1,counter)                                                                                 # Calculate and print remaining scantime
            data.new_block()
        
        agilent.set_status('off')
        
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()
        
    # 2D scan of one dac vs 2 others; the 2 dacs that are stepped together can have arbitrary start and end values
    def _dac_vs_2dacs(self,xname,dacx,xstart,xend,xsteps,yname,dacy,ystart,yend,mname,dacm,mstart,mend,ymsteps):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        y_vector = linspace(ystart,yend,ymsteps+1)
        z_vector = [0]
        
        data = self.create_data(x_vector,xname,dacx,y_vector,yname,dacy,z_vector,'none','z_parameter')                                          # create data file, spyview metafile, copy script
        
        # Define sweep vector for other dac
        m_vector = linspace(mstart,mend,ymsteps+1)
        
        counter = 0
        
        for i in arange(len(y_vector)):
            [starttime, counter] = timetrack.start(counter)
            tstart = timetrack.time()
            y = y_vector[i]
            m = m_vector[i]
            ivvi.set(dacy,y)
            ivvi.set(dacm,m)
            ivvi.set(dacx,x_vector[0.0])
            #T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature 
            qt.msleep(delay1)                                                                                                                   # use explained at the bottom of the script
            
            for x in x_vector:
                datavalues = self.take_data(dacx,x)                                                                                             # Go to next sweep value and take data
                data.add_data_point(x,y,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],datavalues[5],datavalues[6],datavalues[7],datavalues[8],datavalues[9],datavalues[10])                           # write datapoint into datafile
            
            timetrack.remainingtime(starttime,ymsteps+1,counter)                                                                                # Calculate and print remaining scantime
            data.new_block()
            
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()
    
    
    # 2D scan of one dac vs 3 others; the 3 dacs that are stepped together can have arbitrary start and end values
    def _dac_vs_3dacs(self,xname,dacx,xstart,xend,xsteps,yname,dacy,ystart,yend,mname,dacm,mstart,mend,nname,dacn,nstart,nend,ymnsteps):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        y_vector = linspace(ystart,yend,ymnsteps+1)
        z_vector = [0]
        
        data = self.create_data(x_vector,xname,dacx,y_vector,yname,dacy,z_vector,'none','z_parameter')                                          # create data file, spyview metafile, copy script
        
        # Define sweep vector for other dacs
        m_vector = linspace(mstart,mend,ymnsteps+1)
        n_vector = linspace(nstart,nend,ymnsteps+1)
        
        counter = 0
        
        for i in arange(len(y_vector)):
            [starttime, counter] = timetrack.start(counter)
            tstart = timetrack.time()
            y = y_vector[i]
            m = m_vector[i]
            n = n_vector[i]
            ivvi.set(dacy,y)
            ivvi.set(dacm,m)
            ivvi.set(dacn,n)
            ivvi.set(dacx,x_vector[0])
            T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature 
            qt.msleep(delay1)                                                                                                                   # use explained at the bottom of the script
            
            for x in x_vector:
                datavalues = self.take_data(dacx,x)                                                                                             # Go to next sweep value and take data
                data.add_data_point(x,y,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],T_mc,0,0,0,0,0)                           # write datapoint into datafile
            
            timetrack.remainingtime(starttime,ymnsteps+1,counter)                                                                               # Calculate and print remaining scantime
            data.new_block()
            
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()
    
    
    # 2D scan of one dac vs 4 others; the 4 dacs that are stepped together can have arbitrary start and end values
    def _dac_vs_4dacs(self,xname,dacx,xstart,xend,xsteps,yname,dacy,ystart,yend,mname,dacm,mstart,mend,nname,dacn,nstart,nend,pname,dacp,pstart,pend,ymnpsteps):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        y_vector = linspace(ystart,yend,ymnpsteps+1)
        z_vector = [0]
        
        data = self.create_data(x_vector,xname,dacx,y_vector,yname,dacy,z_vector,'none','z_parameter')                                          # create data file, spyview metafile, copy script
        
        # Define sweep vector for other dacs
        m_vector = linspace(mstart,mend,ymnpsteps+1)
        n_vector = linspace(nstart,nend,ymnpsteps+1)
        p_vector = linspace(pstart,pend,ymnpsteps+1)
        
        counter = 0
        
        for i in arange(len(y_vector)):
            [starttime, counter] = timetrack.start(counter)
            tstart = timetrack.time()
            y = y_vector[i]
            m = m_vector[i]
            n = n_vector[i]
            p = p_vector[i]
            ivvi.set(dacy,y)
            ivvi.set(dacm,m)
            ivvi.set(dacn,n)
            ivvi.set(dacp,p)
            ivvi.set(dacx,x_vector[0])
            T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature
            qt.msleep(delay1)                                                                                                                   # use explained at the bottom of the script
            
            for x in x_vector:
                datavalues = self.take_data(dacx,x)                                                                                             # Go to next sweep value and take data
                data.add_data_point(x,y,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],T_mc)                           # write datapoint into datafile
            
            timetrack.remainingtime(starttime,ymnpsteps+1,counter)                                                                              # Calculate and print remaining scantime
            data.new_block()
            
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()

    def _dac_vs_5dacs(self,xname,dacx,xstart,xend,xsteps,yname,dacy,ystart,yend,mname,dacm,mstart,mend,nname,dacn,nstart,nend,pname,dacp,pstart,pend,qname,dacq,qstart,qend,ymnpsteps):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        y_vector = linspace(ystart,yend,ymnpsteps+1)
        z_vector = [0]
        
        data = self.create_data(x_vector,xname,dacx,y_vector,yname,dacy,z_vector,'none','z_parameter')                                          # create data file, spyview metafile, copy script
        
        # Define sweep vector for other dacs
        m_vector = linspace(mstart,mend,ymnpsteps+1)
        n_vector = linspace(nstart,nend,ymnpsteps+1)
        p_vector = linspace(pstart,pend,ymnpsteps+1)
        q_vector = linspace(qstart,qend,ymnpsteps+1)
		
        counter = 0
        
        for i in arange(len(y_vector)):
            [starttime, counter] = timetrack.start(counter)
            tstart = timetrack.time()
            y = y_vector[i]
            m = m_vector[i]
            n = n_vector[i]
            p = p_vector[i]
            q = q_vector[i]
            ivvi.set(dacy,y)
            ivvi.set(dacm,m)
            ivvi.set(dacn,n)
            ivvi.set(dacp,p)
            ivvi.set(dacq,q)
            ivvi.set(dacx,x_vector[0])
            T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature
            qt.msleep(delay1)                                                                                                                   # use explained at the bottom of the script
            
            for x in x_vector:
                datavalues = self.take_data(dacx,x)                                                                                             # Go to next sweep value and take data
                data.add_data_point(x,y,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],T_mc)                           # write datapoint into datafile
            
            timetrack.remainingtime(starttime,ymnpsteps+1,counter)                                                                              # Calculate and print remaining scantime
            data.new_block()
            
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()
        
        
    # 2D scan of 2dacs vs 5 dacs. Added by Ludo. Still work in progress
    def _2dacs_vs_5dacs(self,xname,dacx,xstart,xend,kname,dack,kstart,kend,xksteps,yname,dacy,ystart,yend,mname,dacm,mstart,mend,nname,dacn,nstart,nend,pname,dacp,pstart,pend,qname,dacq,qstart,qend,ymnpsteps):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xksteps+1)
        y_vector = linspace(ystart,yend,ymnpsteps+1)
        z_vector = [0]
        
        data = self.create_data(x_vector,xname,dacx,y_vector,yname,dacy,z_vector,'none','z_parameter')                                          # create data file, spyview metafile, copy script
        
        # Define sweep vector for other dacs
        k_vector = linspace(kstart,kend,xksteps+1)
        m_vector = linspace(mstart,mend,ymnpsteps+1)
        n_vector = linspace(nstart,nend,ymnpsteps+1)
        p_vector = linspace(pstart,pend,ymnpsteps+1)
        q_vector = linspace(qstart,qend,ymnpsteps+1)
		
        counter = 0
        
        for i in arange(len(y_vector)):
            [starttime, counter] = timetrack.start(counter)
            tstart = timetrack.time()
            y = y_vector[i]
            m = m_vector[i]
            n = n_vector[i]
            p = p_vector[i]
            q = q_vector[i]
            ivvi.set(dacy,y)
            ivvi.set(dacm,m)
            ivvi.set(dacn,n)
            ivvi.set(dacp,p)
            ivvi.set(dacq,q)
            ivvi.set(dacx,x_vector[0])
            T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature
            qt.msleep(delay1)                                                                                                                   # use explained at the bottom of the script
            
            for x in x_vector:
                datavalues = self.take_data(dacx,x)                                                                                             # Go to next sweep value and take data
                data.add_data_point(x,y,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],T_mc)                           # write datapoint into datafile
            
            timetrack.remainingtime(starttime,ymnpsteps+1,counter)                                                                              # Calculate and print remaining scantime
            data.new_block()
            
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()        
    
    # 2D scan of one dac vs another one n times repeated, n = repetions
    def _repeat_dac_vs_dac(self,xname,dacx,xstart,xend,xsteps,yname,dacy,ystart,yend,ysteps,repetions):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        y_vector = linspace(ystart,yend,ysteps+1)
        z_vector = linspace(1,repetions,repetions)
        
        data = self.create_data(x_vector,xname,dacx,y_vector,yname,dacy,z_vector,'n','repetions')                                               # create data file, spyview metafile, copy script
        
        counter = 0
        numloops = repetions*(ysteps+1)
        
        for i in arange(repetions):
            ivvi.set(dacy,y[0])
            ivvi.set(dacx,x[0])
           
            for y in y_vector:
                [starttime, counter] = timetrack.start(counter)
                tstart = timetrack.time()
                ivvi.set(dacy,y)
                ivvi.set(dacx,x_vector[0])
                T_mc = self.read_T()                                                                                                            # Read out mixing chamber temperature 
                qt.msleep(delay1)                                                                                                               # use explained at the bottom of the script
            
                for x in x_vector:
                    datavalues = self.take_data(dacx,x)                                                                                         # Go to next sweep value and take data
                    data.add_data_point(x,y,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],T_mc)                       # write datapoint into datafile
            
                timetrack.remainingtime(starttime,numloops,counter)                                                                             # Calculate and print remaining scantime
                data.new_block()
            
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()
        
    
    # 2D scan, every sweep consists of 2 times a dac sweep (first one from start to end of the sweep vector, 2nd one backwards from end to start of the sweep vector), this is done as a function of another dac.
    def _fwdbwd_dac_vs_dac(self,xname,dacx,xstart,xend,xsteps,yname,dacy,ystart,yend,ysteps):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        y_vector = linspace(ystart,yend,ysteps+1)
        z_vector = [0]        
        
        data_fwd = self.create_data(x_vector,xname,dacx,y_vector,yname,dacy,z_vector,'none','z_parameter')                                      # create data file, spyview metafile, copy script for fwd sweep direction
        x_vector = x_vector[::-1]                                                                                                               # reverse direction of sweep vector to ensure proper spyview metafile
        data_bwd = self.create_data(x_vector,xname,dacx,y_vector,yname,dacy,z_vector,'none','z_parameter')                                      # create data file, spyview metafile, copy script for bwd sweep direction
        x_vector = x_vector[::-1]                                                                                                               # reverse direction sweep vector back to original
        
        
        counter = 0
        numloops = ysteps+1
        
        for y in y_vector:
            [starttime, counter] = timetrack.start(counter)
            tstart=timetrack.time();
            ivvi.set(y_parameter,y)
            T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature
            
            for data_loop in (data_fwd, data_bwd):
            
                for x in x_vector:
                    datavalues = self.take_data(dacx,x)                                                                                                      # Go to next sweep value and take data
                    data.add_data_point(x,y,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],T_mc)                       # write datapoint into datafile
                    
                data_loop.new_block()
                
                # Toggle forward / backward by reversing vector
                x_vector = x_vector[::-1]
            
            tstop = timetrack.time()
            timetrack.remainingtime(starttime,numloops,counter)                                                                                 # Calculate and print remaining scantime
            
        for data_loop in (data_fwd, data_bwd):
            data_loop._write_settings_file()                                                                                                    # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
            data_loop.close_file()
            
        qt.mend()

        
    ############### 3D scans ########################
    
    # 3D scan of one dac vs another dac vs another dac
    def _dac_vs_dac_vs_dac(self,xname,dacx,xstart,xend,xsteps,yname,dacy,ystart,yend,ysteps,zname,dacz,zstart,zend,zsteps):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        y_vector = linspace(ystart,yend,ysteps+1)
        z_vector = linspace(zstart,zend,zsteps+1)
        
        data = self.create_data(x_vector,xname,dacx,y_vector,yname,dacy,z_vector,zname,dacz)                                                    # create data file, spyview metafile, copy script
        
        counter = 0
        numloops = (zsteps+1)*(ysteps+1)
        
        for z in z_vector:
            ivvi.set(dacz,z)
            
            for y in y_vector:
                [starttime, counter] = timetrack.start(counter)
                tstart = timetrack.time()
                ivvi.set(dacy,y)
                ivvi.set(dacx,x_vector[0])
                T_mc = self.read_T()                                                                                                            # Read out mixing chamber temperature 
                qt.msleep(delay1)                                                                                                               # use explained at the bottom of the script
            
                for x in x_vector:
                    datavalues = self.take_data(dacx,x)                                                                                         # Go to next sweep value and take data
                    data.add_data_point(x,y,z,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],T_mc)                       # write datapoint into datafile
            
                timetrack.remainingtime(starttime,numloops,counter)                                                                             # Calculate and print remaining scantime
                data.new_block()
            
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()

        
    ############### Scans involving other equipment (magnet supply, rf source) #################




    def _take_data_for_rampBX(self,B_start,B_end): #Hao added, not compatible with Janis
        qt.mstart()

        # Create sweep vectors
        x_vector = linspace(B_start,B_end,900)
        y_vector = [0]
        z_vector = [0]

        data = self.create_data(x_vector,'B','magnet',y_vector,'none','y_parameter',z_vector,'none','z_parameter')                                # create data file, spyview metafile, copy script

        qt.msleep(delay1)
        magnet.rampToX(B_end)
        while True:
            qt.msleep(0.02)
            datavalues = self.take_data('dac8',0.0)
            T_mc = self.read_T()
            B_Now_in_T=magnet.get_fieldX()                                                                                                              # Read out mixing chamber temperature
            data.add_data_point(B_Now_in_T,0,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],datavalues[5],datavalues[6],datavalues[7],T_mc)
            if abs(magnet.get_fieldX()-B_end) < 0.005:
                return

        data.new_block()
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()


    #added by Francesco for conductance measurement as a function of field
    def _take_data_for_rampBZ(self,B_start,B_end):
        qt.mstart()

        # Create sweep vectors
        x_vector = linspace(B_start,B_end,2000)
        y_vector = [0]
        z_vector = [0]

        data = self.create_data(x_vector,'B','magnet',y_vector,'none','y_parameter',z_vector,'none','z_parameter')


        qt.msleep(delay1)

        triton.set_field('CART',0,0,B_start)

        while triton.get_magnetstatus()!="IDLE":
                qt.msleep(0.02)
                print triton.get_magnetstatus()
                print 'Setting starting point'
                qt.msleep(0.02)


        triton.set_field('CART',0,0,B_end)






        while True:
            qt.msleep(0.02)
            datavalues = self.take_data('dac15',0.0)
            T_mc = self.read_T()
            B_Now_in_T= triton.get_Magnet_C3()                                                                                                          # Read out mixing chamber temperature
            data.add_data_point(B_Now_in_T,0,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],datavalues[5],datavalues[6],T_mc,0,0,0)
            if abs(triton.get_Magnet_C3()-B_end) < 0.001:
                return

        data.new_block()
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()

    
    # 1D sweep of magnet     
    def _magnet_sweep_X(self,Bstart,Bend,Bsteps):
        qt.mstart()

        # Create sweep vectors
        B_vector = linspace(Bstart,Bend,Bsteps+1)
        y_vector = [0]
        z_vector = [0]
        
        data = self.create_data(B_vector,'B(T)','triton',y_vector,'none','y_parameter',z_vector,'none','z_parameter')                                # create data file, spyview metafile, copy script
        
        triton.set_field('CART',0,0,Bstart)
        triton.set_Magnet_Sweeprate(0.010)

        while triton.get_magnetstatus()!="IDLE":
                qt.msleep(0.02)
                print triton.get_magnetstatus()
                qt.msleep(0.02)
        
        #set end field, do not wait till finished
        triton.set_field('CART',Bend,0,0)
        print("final field set")
        Bx=Bstart
        qt.msleep(1)
        bsweep = 1
        while bsweep:
            #extract field
            #print("call field")
            #fieldval = re.findall("[-+]?\d*\.\d+|[-+]?\d+", triton.get_field())
                        
            #store field (units T)
            #Bx = float(fieldval[0])
            #By = float(fieldval[1])
            #Bz = float(fieldval[2])
            
            Bx = triton.get_Magnet_C1()
            #print(Bx)
            
            #qt.msleep(0.25)
            
            #print("reading data")
            #T_mc = self.read_T()
            datavalues = self.take_data('dac16',0.0)
            data.add_data_point(Bx,0,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],datavalues[5],datavalues[6],datavalues[7],datavalues[8],datavalues[9],datavalues[10])
            
            if abs(Bx - Bend) < 0.001:
               bsweep = 0
                
         
            #qt.msleep(.25)
            
            #print("checking for magnetstatus")
            #get status
            #if triton.get_magnetstatus() == 'IDLE':
            #    break

        data.new_block()
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()
        
    def _magnet_sweep_Z(self,Bstart,Bend,Bsteps):
        qt.mstart()

        # Create sweep vectors
        B_vector = linspace(Bstart,Bend,Bsteps+1)
        y_vector = [0]
        z_vector = [0]
        
        data = self.create_data(B_vector,'B(T)','triton',y_vector,'none','y_parameter',z_vector,'none','z_parameter')                                # create data file, spyview metafile, copy script
        
        #set field in x-direction, wait till finished
        triton.set_field('CART',0,0,Bstart)
        triton.set_Magnet_Sweeprate(0.002)

        while triton.get_magnetstatus()!="IDLE":
                qt.msleep(0.02)
                print triton.get_magnetstatus()
                qt.msleep(0.02)
        
        print("waiting 5s to set endfield")
        qt.msleep(5)
        
        #set end field, do not wait till finished
        triton.set_field('CART',0,0,Bend)
        print("final field set")
        Bz=Bstart
        qt.msleep(1)
        bsweep = 1
        while bsweep:
            #extract field
            #print("call field")
            #fieldval = re.findall("[-+]?\d*\.\d+|[-+]?\d+", triton.get_field())
                        
            #store field (units T)
            #Bx = float(fieldval[0])
            #By = float(fieldval[1])
            #Bz = float(fieldval[2])
            
            Bz = float(triton.get_Magnet_C3())
            #print(Bx)
            
            #qt.msleep(0.25)
            
            #print("reading data")
            #T_mc = self.read_T()
            datavalues = self.take_data('dac15',0.0)
            data.add_data_point(Bz,0,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],datavalues[5],datavalues[6],datavalues[7],datavalues[8],datavalues[9],datavalues[10],0,0,0)
            
            if abs(Bz - Bend) < 0.001:
               bsweep = 0
                
         
            #qt.msleep(.25)
            
            #print("checking for magnetstatus")
            #get status
            #if triton.get_magnetstatus() == 'IDLE':
            #    break

        data.new_block()
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()
    
    
        # sweep of magnet 2D    
    def _magnet2D_sweep(self,Bstart,Bend,Bsteps,alpha):
        qt.mstart()
        
        #RampRateX = 0.00348                                                                                                                 #Ramp rates in T/sec. delay time after sweeping is adjusted accordingly
        #RampRateY = 0.00149
        #RampRateZ = 0.004
        # Create sweep vectors
        B_vector = linspace(Bstart,Bend,Bsteps+1)
        y_vector = [0]
        z_vector = [0]
        
        data = self.create_data(B_vector,'B(T)','triton',y_vector,'none','y_parameter',z_vector,'none','z_parameter')                                # create data file, spyview metafile, copy script
        
        for B in B_vector:
            #[starttime, counter] = timetrack.start(counter)
            #tstart = timetrack.time()
               
            T_mc = self.read_T()                                                                                                               # Read out mixing chamber temperature             
            qt.msleep(delay3)                                                                                                                   # use explained at the bottom of the script            
            datavalues = self.take_data('dac15',0.0)        #dac that we don't use                                                                                     # Take data at fixed value given by dataval
            data.add_data_point(B,0,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],T_mc)                           # write datapoint into datafile

            #timetrack.remainingtime(starttime,Bsteps+1,counter)                                                                                 # Calculate and print remaining scantime
            data.new_block()
            
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()


    #added by Francesco, fast direction (x) is the magnetic field. slow direction (y) is a dac.
    #used for measuring AB oscillations for different gates setting - need to think on it.
    def _magnetZ_vs_dac(self,B_start,B_end,yname,dacy,ystart,yend,ysteps):
        qt.mstart()

        # Create sweep vectors
        x_vector = linspace(B_start,B_end,15000)
        y_vector = linspace(ystart,yend,ysteps)
        z_vector = [0]

        data = self.create_data(x_vector,'B','magnet',y_vector,yname,dacy,z_vector,'none','z_parameter')

        triton.set_Magnet_Sweeprate(0.003)
        qt.msleep(delay1)

        triton.set_field('CART',0,0,B_start)

        while triton.get_magnetstatus()!="IDLE":
                qt.msleep(0.02)
                print triton.get_magnetstatus()
                print 'Setting starting point'
                qt.msleep(0.02)


        triton.set_Magnet_Sweeprate(0.003)
        triton.set_field('CART',0,0,B_end)






        while True:
            qt.msleep(0.02)
            datavalues = self.take_data('dac15',0.0)
            T_mc = self.read_T()
            B_Now_in_T= triton.get_Magnet_C3()                                                                                                          # Read out mixing chamber temperature
            data.add_data_point(B_Now_in_T,0,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],datavalues[5],datavalues[6],T_mc,0,0,0)
            if abs(triton.get_Magnet_C3()-B_end) < 0.001:
                return

        data.new_block()
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()


    # To scan a dac as a function of B
    def _dac_vs_magnet(self,xname,dacx,xstart,xend,xsteps,Bstart,Bend,Bsteps,dir,alpha,phi,Boffset,OffsetAlpha,Bx,By=0,Bz=0):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        B_vector = linspace(Bstart,Bend,Bsteps+1)
        z_vector = [0]
        
        data = self.create_data(x_vector,xname,dacx,B_vector,'B (T)','triton',z_vector,'none','z_parameter')                                     # create data file, spyview metafile, copy script
              
        counter = 0
        
        for B in B_vector:
            [starttime, counter] = timetrack.start(counter)
            tstart = timetrack.time()
            if dir == 'X':
                triton.set_field('CART',B,By,Bz)
            elif dir == 'Y':
                triton.set_field('CART',Bx,B,Bz)
            elif dir == 'Z':
                triton.set_field('CART',Bx,By,B)
            elif dir == 'A':               #field perpendicular to the substrate. alignment angle is alpha
                # if alpha != magnet.get_alpha()
                    # magnet.set_alpha(alpha)
                # if phi != magnet.get_phi()
                    # magnet.set_phi(phi)
                # if B != magnet.get_field()    
                    # magnet.set_field(B)
                #if Boffset != 0
                    #Bxy = B*sin(phi*pi/180.0)
                    #alpha = alpha - (180.0/pi)*arctan(Boffset/Bxy))
                    #B = copysign(sqrt(B^2+Boffset^2),B)
                #magnet.set_field_XYZ(B,alpha,phi,Boffset,OffsetAlpha)               
                triton.set_field_wait('SPH', B , alpha*pi/180.0 , phi*pi/180.0)
            while triton.get_magnetstatus()!="IDLE":
                qt.msleep(0.02)
                print triton.get_magnetstatus()
                qt.msleep(0.02)
            #waiting time before measuring, Francesco
            qt.msleep(0.1)
            ivvi.set(dacx,x_vector[0])
            T_mc = self.read_T()# Read out mixing chamber temperature
            qt.msleep(delay3)                                                                                                                   # use explained at the bottom of the script
            
            for x in x_vector:
                datavalues = self.take_data(dacx,x)                                                                                             # Go to next sweep value and take data
                data.add_data_point(x,B,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],datavalues[5],datavalues[6], T_mc, 0, 0, 0)                           # write datapoint into datafile
            
            timetrack.remainingtime(starttime,Bsteps+1,counter)
            # Calculate and print remaining scantime
            data.new_block()
            
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()
 

    def _dac_vs_magnet2D(self,xname,dacx,xstart,xend,xsteps,Bstart,Bend,Bsteps,alpha):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        B_vector = linspace(Bstart,Bend,Bsteps+1)
        z_vector = [0]
        
        data = self.create_data(x_vector,xname,dacx,B_vector,'B (T)','triton',z_vector,'none','z_parameter')                                     # create data file, spyview metafile, copy script
        
        counter = 0
        
        magnet.set_alpha(alpha)
        
        for B in B_vector:
            [starttime, counter] = timetrack.start(counter)
            tstart = timetrack.time()
            magnet.set_field(B)               
            ivvi.set(dacx,x_vector[0])
            T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature 
            qt.msleep(delay3)                                                                                                                   # use explained at the bottom of the script
            
            for x in x_vector:
                datavalues = self.take_data(dacx,x)                                                                                             # Go to next sweep value and take data
                data.add_data_point(x,B,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],T_mc)                           # write datapoint into datafile
            
            timetrack.remainingtime(starttime,Bsteps+1,counter)                                                                                 # Calculate and print remaining scantime
            data.new_block()
            
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()
        
    # To sweep a dac as a function of B, every sweep consists of 2 times a dac sweep (first one from start to end of the sweep vector, 2nd one backwards from end to start of the sweep vector).
    def _fwdbwd_dac_vs_B(self,xname,dacx,xstart,xend,xsteps,Bstart,Bend,Bsteps):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        B_vector = linspace(Bstart,Bend,Bsteps+1)
        z_vector = [0]        
        
        data_fwd = self.create_data(x_vector,xname,dacx,B_vector,'B (T)','kepco',z_vector,'none','z_parameter')                                 # create data file, spyview metafile, copy script for fwd sweep direction
        x_vector = x_vector[::-1]                                                                                                               # reverse direction of sweep vector to ensure proper spyview metafile
        data_bwd = self.create_data(x_vector,xname,dacx,B_vector,'B (T)','kepco',z_vector,'none','z_parameter')                                 # create data file, spyview metafile, copy script for bwd sweep direction
        x_vector = x_vector[::-1]                                                                                                               # reverse direction sweep vector back to original
        
        
        counter = 0
        numloops = ysteps+1
        
        for B in B_vector:
            [starttime, counter] = timetrack.start(counter)
            tstart=timetrack.time();
            kepco.set_B(B)                                                                                                                      # Specific for kepco as magnet supply
            T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature
            
            for data_loop in (data_fwd, data_bwd):
            
                for x in x_vector:
                    datavalues = self.take_data(dacx,x)                                                                                         # Go to next sweep value and take data
                    data.add_data_point(x,B,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],T_mc)                       # write datapoint into datafile
                    
                data_loop.new_block()
                
                # Toggle forward / backward by reversing vector
                x_vector = x_vector[::-1]
            
            tstop = timetrack.time()
            timetrack.remainingtime(starttime,numloops,counter)                                                                                 # Calculate and print remaining scantime
            
        for data_loop in (data_fwd, data_bwd):
            data_loop._write_settings_file()                                                                                                    # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
            data_loop.close_file()
            
        qt.mend()
        
    
    # Scan a dac as a function of B, while at the same time another dac is stepped linearly, this allows to correct a bit for features affected by both gate and B field so they don't move much.
        
    def _dac_vs_dacmagnet(self,xname,dacx,xstart,xend,xsteps,yname,dacy,ystart,yend,Bstart,Bend,Bsteps):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        B_vector = linspace(Bstart,Bend,Bsteps+1)
        z_vector = [0]
        
        data = self.create_data(x_vector,xname,dacx,B_vector,'B (T)','kepco',z_vector,'none','z_parameter')                                     # create data file, spyview metafile, copy script
        
        # Create sweep vector for the dac that needs to be stepped together with B
        y_vector = linspace(ystart,yend,Bsteps+1)
        
        counter = 0
        
        for i in arange(len(B_vector)):
            [starttime, counter] = timetrack.start(counter)
            tstart = timetrack.time()
            B = B_vector[i]
            y = y_vector[i]
            kepco.set_B(B)                                                                                                                      # Specific for kepco as magnet supply
            ivvi.set(dacy,y)
            ivvi.set(dacx,x_vector[0])
            T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature 
            qt.msleep(delay1)                                                                                                                   # use explained at the bottom of the script
            
            for x in x_vector:
                datavalues = self.take_data(dacx,x)                                                                                             # Go to next sweep value and take data
                data.add_data_point(x,B,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],T_mc)                           # write datapoint into datafile
            
            timetrack.remainingtime(starttime,Bsteps+1,counter)                                                                                 # Calculate and print remaining scantime
            data.new_block()
            
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()
            
    # Scan a dac as a function of B, while at the same time another dac is stepped linearly, written with an Ic vs B vs TG scan in mind. Ic is stopped whenever a certain measured bias voltage is reached

    def _dac_vs_dacmagnet_threshold(self,xname,dacx,yname,dacy,xstart,xend,xsteps,ystart,yend,ysteps,Bstart,Bend,Bsteps,dir,alpha,phi,Boffset,OffsetAlpha,V_threshold,Bx,By=0,Bz=0):
        qt.mstart()
            
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        y_vector = linspace(ystart,yend,ysteps+1)
        B_vector = linspace(Bstart,Bend,Bsteps+1)
            
        data = self.create_data(x_vector,xname,dacx,y_vector,yname,dacy,B_vector,'triton','B_field')                                      # create data file, spyview metafile, copy script
            
        counter = 0
        numloops = (ysteps+1)*(Bsteps+1)
        
        for y in y_vector:
            ivvi.set(dacy,y)
            #T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature 
            qt.msleep(delay1)   
            triton.set_Magnet_Sweeprate(0.005)#T/min
            triton.set_field_wait('CART',Bx,By,Bz)
            triton.set_Magnet_Sweeprate(0.0005)#T/min
            for B in B_vector:
                [starttime, counter] = timetrack.start(counter)
                tstart = timetrack.time()
                if dir == 'X':
                    triton.set_field_wait('CART',B,By,Bz)
                elif dir == 'Y':
                    triton.set_field_wait('CART',Bx,B,Bz)
                elif dir == 'Z':
                    triton.set_field_wait('CART',Bx,By,B) 
                elif dir == 'A':               #field perpendicular to the substrate. alignment angle is alpha
                    # if alpha != magnet.get_alpha()
                        # magnet.set_alpha(alpha)
                    # if phi != magnet.get_phi()
                        # magnet.set_phi(phi)
                    # if B != magnet.get_field()    
                        # magnet.set_field(B)
                    #if Boffset != 0
                        #Bxy = B*sin(phi*pi/180.0)
                        #alpha = alpha - (180.0/pi)*arctan(Boffset/Bxy))
                        #B = copysign(sqrt(B^2+Boffset^2),B)
                    #magnet.set_field_XYZ(B,alpha,phi,Boffset,OffsetAlpha)               
                    #triton.set_field_wait('SPH', B , alpha*pi/180.0 , phi*pi/180.0)
                    ivvi.set(dacx,x_vector[0])
                    #T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature 
                    qt.msleep(delay3)                                                                                                                   # use explained at the bottom of the script
                
                K2 = 0
                V_start = zeros(5)
                for i in range(0,len(V_start)):
                    V_start[i] =  self.take_data(dacx,xstart)[1]
                V_start_ave = mean(V_start)
                
                V_ref =V_start_ave+V_threshold
                for x in x_vector:                
                    if K2<= V_ref:
                        datavalues = self.take_data(dacx,x)                                                                                             # Go to next sweep value and take data
                        K2 = datavalues[1]
                    else:
                        break
                data.add_data_point(x,B,y,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],datavalues[5],datavalues[6],datavalues[7],datavalues[8],datavalues[9],datavalues[10])                           # write datapoint into datafile                        
                timetrack.remainingtime(starttime,numloops,counter)                                                                                 # Calculate and print remaining scantime
                data.new_block()
            
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend() 

        
    def _dac_vs_2magnet_threshold(self,xname,dacx,xstart,xend,xsteps,B1start,B1end,B1steps,B2start,B2end,B2steps,dir,diralpha,phi,Boffset,OffsetAlpha,V_threshold,Bx,By=0,Bz=0):
        # Note, for now only works if B2 = 'Y' and B1 = 'X'
        qt.mstart()
            
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        B1_vector = linspace(B1start,B1end,B1steps+1)
        B2_vector = linspace(B2start,B2end,B2steps+1)
            
        data = self.create_data(x_vector,xname,dacx,B1_vector,'triton','B_perpendicular',B2_vector,'triton','B_inplane')                                      # create data file, spyview metafile, copy script
            
        counter = 0
        numloops = (B1steps+1)*(B2steps+1)
        
        for B2 in B2_vector:
            #T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature 
            qt.msleep(delay1)   
            triton.set_Magnet_Sweeprate(0.01)#T/min
            triton.set_field_wait('CART',B1_vector[0],B2,Bz)
            triton.set_Magnet_Sweeprate(0.0005)#T/min
            for B1 in B1_vector:
                [starttime, counter] = timetrack.start(counter)
                tstart = timetrack.time()
                if dir == 'X':
                    triton.set_field_wait('CART',B1,B2,Bz)
                elif dir == 'Y':
                    triton.set_field_wait('CART',Bx,B,Bz)
                elif dir == 'Z':
                    triton.set_field_wait('CART',Bx,By,B) 
                elif dir == 'A':               #field perpendicular to the substrate. alignment angle is alpha
                    # if alpha != magnet.get_alpha()
                        # magnet.set_alpha(alpha)
                    # if phi != magnet.get_phi()
                        # magnet.set_phi(phi)
                    # if B != magnet.get_field()    
                        # magnet.set_field(B)
                    #if Boffset != 0
                        #Bxy = B*sin(phi*pi/180.0)
                        #alpha = alpha - (180.0/pi)*arctan(Boffset/Bxy))
                        #B = copysign(sqrt(B^2+Boffset^2),B)
                    #magnet.set_field_XYZ(B,alpha,phi,Boffset,OffsetAlpha)               
                    #triton.set_field_wait('SPH', B , alpha*pi/180.0 , phi*pi/180.0)
                    ivvi.set(dacx,x_vector[0])
                    #T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature 
                    qt.msleep(delay3)                                                                                                                   # use explained at the bottom of the script
                
                K2 = 0
                V_start = zeros(5)
                for i in range(0,len(V_start)):
                    V_start[i] =  self.take_data(dacx,xstart)[1]
                V_start_ave = mean(V_start)
                
                V_ref =V_start_ave+V_threshold
                for x in x_vector:                
                    if K2<= V_ref:              
                        datavalues = self.take_data(dacx,x)                                                                                             # Go to next sweep value and take data
                        K2 = datavalues[1]
                    else:
                        break
                data.add_data_point(x,B1,B2,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],datavalues[5],datavalues[6],datavalues[7],datavalues[8],datavalues[9],datavalues[10])                           # write datapoint into datafile                        
                timetrack.remainingtime(starttime,numloops,counter)                                                                                 # Calculate and print remaining scantime
                data.new_block()
            
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend() 
        
    def _dac_vs_2dacmagnet_threshold(self,xname,dacx,yname,dacy,zname,dacz,xstart,xend,xsteps,ystart,yend,zstart,zend,yzsteps,Bstart,Bend,Bsteps,dir,alpha,phi,Boffset,OffsetAlpha,V_threshold,Bx,By=0,Bz=0):
        # Note, only works for B = 'X' direction
        qt.mstart()
            
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        y_vector = linspace(ystart,yend,yzsteps+1)
        z_vector = linspace(zstart,zend,yzsteps+1)
        B_vector = linspace(Bstart,Bend,Bsteps+1)
            
        data = self.create_data(x_vector,xname,dacx,y_vector,yname,dacy,B_vector,'triton','B_field')                                      # create data file, spyview metafile, copy script
            
        counter = 0
        numloops = (yzsteps+1)*(Bsteps+1)
        
        for j in arange(len(y_vector)):
            y = y_vector[j]
            z = z_vector[j]
            ivvi.set(dacy,y)
            ivvi.set(dacz,z)
            #T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature 
            qt.msleep(delay1)   
            triton.set_Magnet_Sweeprate(0.005)#T/min
            triton.set_field_wait('CART',Bstart,By,Bz)
            triton.set_Magnet_Sweeprate(0.0005)#T/min
            for B in B_vector:
                [starttime, counter] = timetrack.start(counter)
                tstart = timetrack.time()
                if dir == 'X':
                    triton.set_field_wait('CART',B,By,Bz)
                elif dir == 'Y':
                    triton.set_field_wait('CART',Bx,B,Bz)
                elif dir == 'Z':
                    triton.set_field_wait('CART',Bx,By,B) 
                elif dir == 'A':               #field perpendicular to the substrate. alignment angle is alpha
                    # if alpha != magnet.get_alpha()
                        # magnet.set_alpha(alpha)
                    # if phi != magnet.get_phi()
                        # magnet.set_phi(phi)
                    # if B != magnet.get_field()    
                        # magnet.set_field(B)
                    #if Boffset != 0
                        #Bxy = B*sin(phi*pi/180.0)
                        #alpha = alpha - (180.0/pi)*arctan(Boffset/Bxy))
                        #B = copysign(sqrt(B^2+Boffset^2),B)
                    #magnet.set_field_XYZ(B,alpha,phi,Boffset,OffsetAlpha)               
                    #triton.set_field_wait('SPH', B , alpha*pi/180.0 , phi*pi/180.0)
                    ivvi.set(dacx,x_vector[0])
                    #T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature 
                    qt.msleep(delay3)                                                                                                                   # use explained at the bottom of the script
            
                K2 = 0
                V_start = zeros(5)
                for i in range(0,len(V_start)):
                    V_start[i] =  self.take_data(dacx,xstart)[1]
                V_start_ave = mean(V_start)
                
                V_ref =V_start_ave+V_threshold
                for x in x_vector:                
                    if K2<= V_ref:
                        datavalues = self.take_data(dacx,x)                                                                                             # Go to next sweep value and take data
                        K2 = datavalues[1]
               
                    else:
                        break
                data.add_data_point(x,B,y,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],datavalues[5],datavalues[6],datavalues[7],datavalues[8],datavalues[9],datavalues[10])                           # write datapoint into datafile                        
                timetrack.remainingtime(starttime,numloops,counter)                                                                                 # Calculate and print remaining scantime
                data.new_block()
            
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend() 
      
   
    # dac vs rf power, to do power dependence of Shapiro steps. rf power is stepped with non-uniform steps following P^2 ~ Vrf. Function only works well if you start the sweep at the lowest power.
    def _dac_vs_rfpower2(self,xname,dacx,xstart,xend,xsteps,Pstart,Pend,Psteps,freq):
        qt.mstart()
        
        # Create sweep vectors
        x_vector = linspace(xstart,xend,xsteps+1)
        
        # P vector needs to have quadratic shape
        yend = (Pend-Pstart)*(Pend-Pstart)
        y_vector = linspace(0,yend,Psteps+1)
        P_vector = sqrt(y_vector)+Pstart
        
        z_vector = [0]
        
        data = self.create_data(x_vector,xname,dacx,P_vector,'Prf (dB)','MW source',z_vector,'none','z_parameter')                               # create data file, spyview metafile, copy script
        
        counter = 0
        
        # initialization of MW source
        mw.set_power(-20.0)
        mw.set_frequency(freq)
        mw.set_status('ON')
        
        for P in P_vector:
            [starttime, counter] = timetrack.start(counter)
            tstart = timetrack.time()
            mw.set_power(P)                                                                                                                     # Set mw power
            ivvi.set(dacx,x_vector[0])
            T_mc = self.read_T()                                                                                                                # Read out mixing chamber temperature
            qt.msleep(delay1)                                                                                                                   # use explained at the bottom of the script
            
            for x in x_vector:
                datavalues = self.take_data(dacx,x)                                                                                             # Go to next sweep value and take data
                data.add_data_point(x,P,0,datavalues[0],datavalues[1],datavalues[2],datavalues[3],datavalues[4],T_mc)                           # write datapoint into datafile
            
            timetrack.remainingtime(starttime,Psteps+1,counter)                                                                                 # Calculate and print remaining scantime
            data.new_block()
        
        mw.set_status('OFF')
        data._write_settings_file()                                                                                                             # Overwrite the settings file created at the beginning, this ensures updating the sweep variable with the latest value
        data.close_file()
        qt.mend()
    
        
        
#################### INITIALIZATION #########################

# DON'T SKIP THIS PART, ITS CRUCIAL FOR PROPER MEASUREMENTS AND DATA PROCESSING!!!

# Gains and ranges
# Please set the gains and ranges before starting measurements. This ensures proper scaling of axis and data in Spyview.
# Make sure that you put the right gain at the right Keithley/Lockin.
GainK1=1e6                 # Gain for Keitley 1 - M1b module
GainK2=1e6                # Gain for Keitley 2 N/A: high voltage source, 1uA/V
#GainK3=1e6                  # Gain keithley 3. leakage current here.
#GainK4=1e6
GainL1=1e6                  # Gain for Lockin 1 - M1b module @ 1M and x1
#GainL2=1e2                  # Gain for Lockin 2 - N/A
#GainL3=1e6
Vrange=1e-3                     # voltage range in V/V - S3b module
#Irange=1e-6                     # current range in A/V - S4m module
#TGrange=15.0                       # Gate range in V/V
BGrange=15.0                      # Gate range in V/V

# Measurement setup:
#       -Current-biased with K1
#S0 settings: 50 KHz; 50 kHz
#S3b settings: (10m) N/A
#S4m settings: 100 nA/V; on; v2=-v4
#S1h settings: 15V/V
#S1f settings:
#M2b settings: 1k, dc
#M1b settings: Low noise; 2=gnd; off; 1M and x1
#M2m settings: 1e0 V/V; DC 
#M0 settings: Out1: 10 kHz; Out2: 10 kHz;
#External LP filter box: NONE (1 kHz on Keithley1-2700; Moving avg.10 times)

print '\nGainK1='+`GainK1`+'\t\tGain for Keithley 1\nGainL1='+`GainL1`+'\t\tGain for Lockin 1\nVrange='+`Vrange`+'\t\tVoltage range in V/V\nBGrange='+`BGrange`+'\t\tGate range in V/V\n'
s=raw_input("Ensure smooth dataprocessing in spyview by checking if gains, Vrange, Irange and BGrange are stored correct. Press enter to continue, press any other key and enter to stop: ")
if s!='':
    print '\nVery kind of you to avoid future data processing problems.'
    time.sleep(3.0)
    sys.exit()

# Please update the number of Keithleys/Lockins used in the measurement, this ensures proper data readout and storage
# Remark: In the program its assumed that if you use a single Keithley, this one is Keithley 1. Same for the lockins.
# To ensure proper data processing, we encourage reading the 'take_data' function above.
Nkeithleys = 1
Nlockins = 1

# Set Keithley averaging
keithley1.set_nplc(5)

#triton.set_Magnet_Sweeprate(0.005)#T/min
keithley1.set_autorange(1)
keithley1.set_averaging(0)





#set dc voltageivvi 
#dcvoltage = 100e-6
#ivvi.set_dac2(100+30)


#Initialization of the lockins
amplitude = 10e-6                                        # Excitation amplitude in V
lockin1.set_sensitivity(18)                               # Check S830 driver for the meaning of this number;  13 = 50 uV,  15 = 200 uV, 16 = 500 uV, 18 = 2mV, 19= 5 mV  24=200 mV
lockin1.set_amplitude(amplitude/(Vrange*1e-2*sqrt(2)))  #10*0.01 mV/V
lockin1.set_tau(8)                                          # 9 = 300 ms  # 8 = 100 ms # 7 = 30 ms  # 6 = 10 ms
lockin1.set_frequency(83.17)

# This delay is needed at the beginning of a sweep, a good value is ~ 10*tau. If you don't use it, first few datapoints of the sweep will be weird. 
# Its safe to put it to 0 for Keithley measurements. 

delay1 = 6 #(10.0 for gate scans)

# Delay after setting dac during sweep, you should at least wait 1.5*tau, longer is better. This avoids integration over the previous dac value.
delay2 = 0.15  #0.1#0.05 #=0.3, (1.0 for gate scans)
#Delay for magnetic field sweeps. In order to prevent excessive hea ting, we wait some time before starting inner loop after the magnetic field settled.
delay3 = 10.0 #10.0

#agilent = qt.instruments.create('agilent', 'HP_8657B', address='GPIB0::7', reset=False)

filename= 'loop2_temperature_scan_450mk'


#################### MEASUREMENTS #########################


m = majorana()




#ivvi.set_parameter_rate('dac1',1,15)
ivvi.set_parameter_rate('dac1',50,1)
ivvi.set_parameter_rate('dac2',1,10)

ivvi.set_parameter_rate('dac3',0.3,50)  #BG
ivvi.set_parameter_rate('dac4',0.5,25)  #T2 x5

#ivvi.set_parameter_rate('dac5',0.1,4)  #PG
ivvi.set_parameter_rate('dac5',0.5,25)  #T1 x5
#ivvi.set_parameter_rate('dac6',0.1,40)  #TG1
#ivvi.set_parameter_rate('dac6',25,1)  #TG1 without amplification x5
ivvi.set_parameter_rate('dac6',0.5,25)  #PG x5
ivvi.set_parameter_rate('dac7',0.5,25)  #EG x5
ivvi.set_parameter_rate('dac8',0.5,25)  #RG x5
#ivvi.set_parameter_rate('dac7',0.2,50)  #TG2 with amplification x5
#ivvi.set_parameter_rate('dac7',25,1)  #TG2 without amplification x5

#ivvi.set_parameter_rate('dac8',0.5,25) #EG
#ivvi.set_parameter_rate('dac9',0.5,25) #RG

ivvi.set_parameter_rate('dac11',5,1)  #T2 x5
ivvi.set_parameter_rate('dac12',0.2,20)  #T2 x5

#ivvi.set_parameter_rate('dac11',25,1)  #T1
#ivvi.set_parameter_rate('dac12',1,20)  #T1*


ivvi.set_parameter_rate('dac13',0.2,20)  #RG x5
ivvi.set_parameter_rate('dac14',0.2,20)  #RG x5

#ivvi.set_parameter_rate('dac13',25,1)  #PG
#ivvi.set_parameter_rate('dac14',1,20)  #PG*

#ivvi.set_parameter_rate('dac15',0.1,4)  #PG
ivvi.set_parameter_rate('dac15',25,1)  #T2
ivvi.set_parameter_rate('dac16',1,20)  #T2*

#biasV

#ivvi.set_dac1(1000)

#m._single_dac_sweep('EG', 'dac7',-1500,0,750)
#m._single_dac_sweep('RG', 'dac8',-800,-550,125)
#m._single_dac_sweep('T2', 'dac4',-240,0,240)
#m._single_dac_sweep('T2', 'dac4',0,-500,500)
#m._single_dac_sweep('T2', 'dac4',-500,0,500)
#m._single_dac_sweep('T1', 'dac5',-2000,0,1000)
#m._single_dac_sweep('PG', 'dac6',-2000,0,1000)
#m._single_dac_sweep('Vbias', 'dac1',1000,0,250)

ivvi.set_dac1(0.0)
ivvi.set_dac4(0)  #T2
ivvi.set_dac8(200) #RG

qt.msleep(600)

m._take_data_for_rampBZ(0.8, 1.0)

m._take_data_for_rampBZ(1.0,0.8)

m._take_data_for_rampBZ(0.8, 1.0)

m._take_data_for_rampBZ(1.0,0.8)

m._take_data_for_rampBZ(0.8, 1.0)

m._take_data_for_rampBZ(1.0,0.8)

m._take_data_for_rampBZ(0.8, 1.0)

m._take_data_for_rampBZ(1.0,0.8)





#m._dac_vs_dac('TG','dac5',-200,500,250, 'BG','dac3',-400,1000,40)

#m._single_dac_sweep('TG', 'dac5',200,1000,600)
#m._single_dac_sweep('BG', 'dac3',1000,0,500)

#ivvi.set_dac1(200)
#m._single_dac_sweep('BG', 'dac3',0,1000,500)
#m._single_dac_sweep('BG', 'dac3',1000,0,500)

#m._single_dac_sweep('BG', 'dac3',1000,0,1000)
#m._dac_vs_dac( 'Vbias','dac1',-1200,+1200,600, 'PG*','dac16',1000,-200,850)

#triton.set_Magnet_Sweeprate(0.010) #T/min

#m._single_dac_sweep('PG*', 'dac16',0,2000,1000)


#m._single_dac_sweep('PG', 'dac16',2000,0,1000)




#m._single_dac_sweep('T2', 'dac13',-500,-1500,300)

#m._single_dac_sweep('BG', 'dac3',0,400,200)
#qt.msleep(4)
#m._single_dac_sweep('PG', 'dac15',400,-800,200)
#qt.msleep(2)
#ivvi.set_dac6(-83) #T1
#ivvi.set_dac7(-73) #T2
#qt.msleep(4)
#m._single_dac_sweep('PG*', 'dac16',2000,-2000,6000)
#ivvi.set_dac6(0) #T1
#qt.msleep(4)
#m._single_dac_sweep('T2', 'dac7',0,-400,200)



#m._dac_vs_dac('PG','dac15',-2000,200,600, 'BG','dac3',0,1000,25)
#m._single_dac_sweep('PG*', 'dac16',0,1000,1000)
#m._dac_vs_dac( 'T1','dac11',0,-600,50, 'T2','dac13',-600,-900,25)
#m._dac_vs_dac( 'Vbias','dac1',-1000,+1000,600, 'PG*','dac16',0,1000,500)

#m._single_dac_sweep('BG', 'dac3',-800,0,250)







#ivvi.set_dac15(-150)

#m._single_dac_sweep('BG', 'dac3',0,200,100)
#qt.msleep(4)
#m._single_dac_sweep('PG*', 'dac16',0,700,350)
#qt.msleep(4)
#m._dac_vs_dac( 'Vbias','dac1',-1000,+1000,500, 'T2','dac14',500,850,30)


#ivvi.set_dac1(1000)
#qt.msleep(4)




#m._single_dac_sweep('T1', 'dac12',-2000,2000,600)
#m._single_dac_sweep('T2', 'dac14',-2000,2000,600)
#m._single_dac_sweep('PG', 'dac16',0,2000,600)





#m._dac_vs_dac('T1','dac6',-100,0,50, 'T2','dac7',-100,0,50)
#m._single_dac_sweep('BG', 'dac3',440,640,300)



#ivvi.set_dac1(1000)

#m._single_dac_sweep('PG', 'dac6',-800,300,1100)
#qt.msleep(300)

#m._single_dac_sweep('PG', 'dac6',-550,950,1100)
#qt.msleep(300)

#m._single_dac_sweep('PG', 'dac6',-550,950,1100)
#qt.msleep(600)


#m._single_dac_sweep('PG', 'dac6',-700,200,900)

#ivvi.set_dac6(200) #T1
#ivvi.set_dac7(200) #T2
#m._single_dac_sweep('T2', 'dac7',200,-800,700)


#m._single_dac_sweep('PG', 'dac13',200,-700, 300 )
#m._single_dac_sweep('PG*', 'dac14',80, -160,200)


#triton.set_Magnet_Sweeprate(0.010) #T/min

'''

m._dac_vs_magnet('Vbias','dac1',
                    -1300,+1300,650,           #xstart, xend, xsteps
                    1.85,3.5,60,      #bstart, bend, bsteps
                    'Z',0,90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,
                    0,0.0,0)                   #Bx_set, By_set, Bz_set

triton.set_Magnet_Sweeprate(0.010) #T/min
triton.set_field('CART',0.0,0, 0.0)
m._dac_vs_magnet('Vbias','dac1',
                    -1300,+1300,650,           #xstart, xend, xsteps
                    0,4.0,80,      #bstart, bend, bsteps
                    'Z',0,90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,
                    0,0.0,0)                   #Bx_set, By_set, Bz_set


m._dac_vs_magnet('PG','dac6',
                    -1000,-990,1,           #xstart, xend, xsteps
                    4.0,5.0,30,      #bstart, bend, bsteps
                    'Z',0,90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,
                    0,0.0,0)                   #Bx_set, By_set, Bz_set

#m._dac_vs_dac('Vbias','dac1',1500,-1500,500, 'PG','dac6',-750,-400,100)


ivvi.set_dac13(200)  #PG
ivvi.set_dac14(250)  #PG*
m._dac_vs_dac('T1', 'dac12', 0, 1200, 30, 'T2', 'dac16',350, 1650, 30)

ivvi.set_dac13(100)  #PG
ivvi.set_dac14(250)  #PG*
m._dac_vs_dac('T1', 'dac12', 0, 1200, 30, 'T2', 'dac16',350, 1650, 30)

ivvi.set_dac13(0)  #PG
ivvi.set_dac14(250)  #PG*
m._dac_vs_dac('T1', 'dac12', 0, 1200, 30, 'T2', 'dac16',350, 1650, 30)


ivvi.set_dac13(-200)  #PG
ivvi.set_dac14(250)  #PG*
m._dac_vs_dac('T1', 'dac12', 0, 1200, 30, 'T2', 'dac16',350, 1650, 30)


ivvi.set_dac13(-600)  #PG
ivvi.set_dac14(250)  #PG*
m._dac_vs_dac('T1', 'dac12', 0, 1200, 30, 'T2', 'dac16',350, 1650, 30)

ivvi.set_dac13(-1000)  #PG
ivvi.set_dac14(250)  #PG*
m._dac_vs_dac('T1', 'dac12', 0, 1200, 30, 'T2', 'dac16',350, 1650, 30)


#triton.set_field('CART',0.0,0.0,5.0)





triton.set_Magnet_Sweeprate(0.025)#T/min
m._dac_vs_magnet('BG','dac3',
                    -300,1000,400,           #xstart, xend, xsteps
                    0,5.0,100,      #bstart, bend, bsteps
                    'Z',0,90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,
                    0,0.0,0)                   #Bx_set, By_set, Bz_set


m._dac_vs_magnet('PG','dac16',
                    80,-500,580,           #xstart, xend, xsteps
                    0,0.9,45,      #bstart, bend, bsteps
                    'Y',0.90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,
                    0,0.0,0)                   #Bx_set, By_set, Bz_set

triton.set_field('CART',0,0,0)

m._dac_vs_magnet('PG','dac16',
                    80,-500,580,           #xstart, xend, xsteps
                    0,2.0,50,      #bstart, bend, bsteps
                    'Z',0,90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,
                    0,0.0,0)                   #Bx_set, By_set, Bz_set



triton.set_field('CART',0,0,0)



# Magnetic field measurements
triton.set_Magnet_Sweeprate(0.04)#T/min
#triton.set_field_wait('CART',0,0,-0.1)
ivvi.set_dac3(0) 
ivvi.set_dac1(1000) # 100 nA bias
m._magnet_sweep_Z(0.5,-0.1,600)

triton.set_Magnet_Sweeprate(0.01)#T/min
triton.set_field_wait('CART',0.03,0,0)
triton.set_field_wait('CART',-0.03,0,0)
qt.msleep(120)
ivvi.set_dac3(4000/15.0)
ivvi.set_dac4(4000/15.0)
triton.set_Magnet_Sweeprate(0.0005)#T/min
m._dac_vs_magnet('Ibias','dac1',
                    0,100,100,           #xstart, xend, xsteps
                    -0.02,0.02,400,      #bstart, bend, bsteps
                    'X',0,90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,
                    0,0.0,0)                   #Bx_set, By_set, Bz_set
ivvi.set_dac1(0)
triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0,0,0)
qt.msleep(120)

triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0.008,0,0)
triton.set_field_wait('CART',-0.008,0,0)
qt.msleep(120)
ivvi.set_dac3(0/15.0)
ivvi.set_dac4(0/15.0)
triton.set_Magnet_Sweeprate(0.0005)#T/min
m._dac_vs_magnet('Ibias','dac1',
                    0,10,100,           #xstart, xend, xsteps
                    -0.008,0.008,50,      #bstart, bend, bsteps
                    'X',0,90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,
                    0,0.0,0)                   #Bx_set, By_set, Bz_set
ivvi.set_dac1(0)
triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0,0,0)
qt.msleep(120)

triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0.02,0,0)
triton.set_field_wait('CART',-0.02,0,0)
qt.msleep(120)
ivvi.set_dac3(-3000/15.0)
ivvi.set_dac4(200/15.0)
triton.set_Magnet_Sweeprate(0.0005)#T/min
m._dac_vs_magnet('Ibias','dac1',
                    0,1000,100,           #xstart, xend, xsteps
                    -0.02,0.02,100,      #bstart, bend, bsteps
                    'X',0,90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,
                    0,0.0,0)                   #Bx_set, By_set, Bz_set
ivvi.set_dac1(0)
triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0,0,0)
qt.msleep(120)

triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0.02,0,0)
triton.set_field_wait('CART',-0.02,0,0)
qt.msleep(120)
ivvi.set_dac3(-3450/15.0)
ivvi.set_dac4(600/15.0)
triton.set_Magnet_Sweeprate(0.0005)#T/min
m._dac_vs_magnet('Ibias','dac1',
                    0,1000,100,           #xstart, xend, xsteps
                    -0.02,0.02,100,      #bstart, bend, bsteps
                    'X',0,90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,
                    0,0.0,0)                   #Bx_set, By_set, Bz_set
ivvi.set_dac1(0)
triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0,0,0)
qt.msleep(120)

triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0.02,0,0)
triton.set_field_wait('CART',-0.02,0,0)
qt.msleep(120)
ivvi.set_dac3(4000/15.0)
ivvi.set_dac4(-1000/15.0)
triton.set_Magnet_Sweeprate(0.0005)#T/min
m._dac_vs_magnet('Ibias','dac1',
                    0,1000,100,           #xstart, xend, xsteps
                    -0.02,0.02,100,      #bstart, bend, bsteps
                    'X',0,90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,
                    0,0.0,0)                   #Bx_set, By_set, Bz_set
ivvi.set_dac1(0)
triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0,0,0)
qt.msleep(120)

triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0.02,0,0)
triton.set_field_wait('CART',-0.02,0,0)
qt.msleep(120)
ivvi.set_dac3(-1500/15.0)
ivvi.set_dac4(0/15.0)
triton.set_Magnet_Sweeprate(0.0005)#T/min
m._dac_vs_magnet('Ibias','dac1',
                    0,1000,100,           #xstart, xend, xsteps
                    -0.02,0.02,100,      #bstart, bend, bsteps
                    'X',0,90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,
                    0,0.0,0)                   #Bx_set, By_set, Bz_set
ivvi.set_dac1(0)
triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0,0,0)
qt.msleep(120)


triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0.02,0,0)
triton.set_field_wait('CART',-0.02,0,0)
triton.set_field_wait('CART',-0.003,0,0)
qt.msleep(120)
ivvi.set_dac3(4000/15.0)
ivvi.set_dac4(4000/15.0)
triton.set_Magnet_Sweeprate(0.0005)#T/min
m._dac_vs_magnet('Ibias','dac1',
                    0,100,100,           #xstart, xend, xsteps
                    -0.005,0.1,500,      #bstart, bend, bsteps
                    'X',0,90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,
                    0,0.0,0)                   #Bx_set, By_set, Bz_set
ivvi.set_dac1(0)
triton.set_Magnet_Sweeprate(0.01)#T/min
triton.set_field_wait('CART',0,0,0)
qt.msleep(120)


triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0.05,0,0)
triton.set_field_wait('CART',-0.005,0,0)
qt.msleep(120)
triton.set_Magnet_Sweeprate(0.0005)#T/min
m._dac_vs_magnet('Ibias','dac1',
                    0,70,200,           #xstart, xend, xsteps
                    -0.005,0.05,1000,      #bstart, bend, bsteps
                    'X',0,90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,
                    0,0.0,0)                   #Bx_set, By_set, Bz_set
ivvi.set_dac1(0)
triton.set_field_wait('CART',0,0,0)
qt.msleep(120)

#Vg = linspace(0,-2.6,2)*1000/15.0
for i in range(0,len(Vg)):
    ivvi.set_dac3(Vg[i])
    m._dac_vs_magnet('Ibias','dac1',
                        -150,150,150,           #xstart, xend, xsteps
                        -0.01,0.01,500,      #bstart, bend, bsteps
                        'X',0,90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,

                        0,0.0,0)                  #Bx_set, By_set, Bz_set 
ivvi.set_dac1(0)
triton.set_field_wait('CART',0,0,0)

triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0.005,0,0)
triton.set_field_wait('CART',-0.005,0,0)
qt.msleep(120)
triton.set_Magnet_Sweeprate(0.001)#T/min
ivvi.set_dac3(-240)
m._dac_vs_dacmagnet_threshold('Ibias','dac1','BG','dac4',
                    0,1000,200,                         #xstart, xend, xsteps
                    0,-240,8,                            #ystart, yend, ysteps
                    -0.005,0.005,100,                         #bstart, bend, bsteps
                    'X',0,90,0,0,                       #mode, theta, Phi, B_offset, alpha_offset,
                    0.002,                               #V_threshold
                    0,0.0,0)                            #Bx_set, By_set, Bz_set  
triton.set_field_wait('CART',0,0,0)
ivvi.set_dac1(0)

triton.set_Magnet_Sweeprate(0.01)#T/min
triton.set_field_wait('CART',0.002,-0.05,0)
triton.set_field_wait('CART',-0.002,-0.05,0)
qt.msleep(120)
ivvi.set_dac3(-5000/15.0)
ivvi.set_dac4(-1290/15.0)
triton.set_Magnet_Sweeprate(0.0005)#T/min
m._dac_vs_2magnet_threshold('Ibias','dac1',              
                    0,60,120,                           #xstart, xend, xsteps
                    -0.002,0.002,100,                   #B1_start,B1_end,B1_steps (perpendicular/sweep)
                    -0.05,0.05,4,                   #B2_start,B2_end,B2_steps (in_plane/step)
                    'X',0,90,0,0,                       #mode, theta, Phi, B_offset, alpha_offset,
                    3e-6,                               #V_threshold
                    0,0,0)                        #Bx_set, By_set, Bz_set  

triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0,0,0)
ivvi.set_dac1(0)

ivvi.set_dac3(0)
ivvi.set_dac4(-1650/15.0)
triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0.01,0,0)
triton.set_field_wait('CART',-0.01,0,0)
qt.msleep(120)
triton.set_Magnet_Sweeprate(0.0005)#T/min
m._dac_vs_magnet('Ibias','dac1',
                    0,25,50,           #xstart, xend, xsteps
                    -0.01,0.01,500,      #bstart, bend, bsteps
                    'X',0,90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,
                    0,0.0,0)                   #Bx_set, By_set, Bz_set
ivvi.set_dac1(0)
triton.set_field_wait('CART',0,0,0)
qt.msleep(120)

triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0.01,0,0)
triton.set_field_wait('CART',-0.01,0,0)
triton.set_field_wait('CART',-0.004,0,0)
qt.msleep(120)
triton.set_Magnet_Sweeprate(0.0005)#T/min
m._dac_vs_magnet('Ibias','dac1',
                    0,800,1600,           #xstart, xend, xsteps
                    -0.004,-0.001,150,      #bstart, bend, bsteps
                    'X',0,90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,
                    0,0.0,0)                   #Bx_set, By_set, Bz_set
ivvi.set_dac1(0)
triton.set_field_wait('CART',0,0,0)
qt.msleep(120)

triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0.01,0,0)
triton.set_field_wait('CART',-0.01,0,0)
triton fill in magnetic field central lobe
qt.msleep(120)
triton.set_Magnet_Sweeprate(0.0005)#T/min
m._dac_vs_magnet('Ibias','dac1',
                    0,700,1250,           #xstart, xend, xsteps
                    -x,x,d,      #bstart, bend, bsteps
                    'X',0,90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,
                    0,0.0,0)                   #Bx_set, By_set, Bz_set
ivvi.set_dac1(0)
triton.set_field_wait('CART',0,0,0)
qt.msleep(120)
+ shapiro below


triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',-0.002,0,0)
triton.set_Magnet_Sweeprate(0.005)#T/min
m._dac_vs_2dacmagnet_threshold('Ibias','dac1','TG','dac3','BG','dac4',
                    0,200,200,                         #xstart, xend, xsteps
                    -0/15.0,-2250/15.0,20            #ystart, yend, ysteps
                    -25/15.0,-25/15.0,20,                #zstart,zend,yzsteps
                    -0.002,0.002,100,                   #bstart, bend, bsteps
                    'X',0,90,0,0,                       #mode, theta, Phi, B_offset, alpha_offset,
                    2e-6,                               #V_threshold
                    0,0.0,0)                            #Bx_set, By_set, Bz_set  
triton.set_field_wait('CART',0,0,0)
ivvi.set_dac1(0)



ivvi.set_dac3(-5000/15.0)
ivvi.set_dac4(-1290/15.0)
qt.msleep(3)
triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0.002,0,0)
triton.set_field_wait('CART',-0.002,0,0)
qt.msleep(120)
triton.set_Magnet_Sweeprate(0.0005)#T/min
m._dac_vs_magnet('Ibias','dac1',
                    0,15,200,           #xstart, xend, xsteps
                    -0.002,0.002,100,      #bstart, bend, bsteps
                    'X',0,90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,
                    0,0.0,0)                   #Bx_set, By_set, Bz_set
ivvi.set_dac1(0)
qt.msleep(120)
triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',-0.002,0,0)
qt.msleep(120)


triton.set_Magnet_Sweeprate(0.005)#T/min
triton.set_field_wait('CART',0.01,0,0)
triton.set_field_wait('CART',-0.01,0,0)
qt.msleep(120)
triton.set_Magnet_Sweeprate(0.0005)#T/min
m._dac_vs_magnet('Ibias','dac1',
                    0,150,250,           #xstart, xend, xsteps
                    -0.01,0.01,500,      #bstart, bend, bsteps
                    'X',0,90,0,0,           #mode, theta, Phi, B_offset, alpha_offset,
                    0,0.0,0)                   #Bx_set, By_set, Bz_set
ivvi.set_dac1(0)
qt.msleep(120)
'''


